@@ -84,6 +84,7 @@
 #include "nsContentErrors.h"
 #include "nsLayoutStatics.h"
 #include "nsIScriptObjectPrincipal.h"
+#include "nsIMultiPartChannel.h"
 
 static const char* kLoadAsData = "loadAsData";
 #define LOADSTR NS_LITERAL_STRING("load")
@@ -878,6 +879,9 @@ nsXMLHttpRequest::NotifyEventListeners(nsIDOMEventListener* aHandler,
     for (PRInt32 i = 0, i_end = aListeners->Count(); i < i_end; ++i) {
       nsIDOMEventListener *listener = aListeners->ObjectAt(i);
       if (listener) {
+        if (NS_FAILED(CheckInnerWindowCorrectness())) {
+          break;
+        }
         listener->HandleEvent(aEvent);
       }
     }
@@ -930,6 +934,18 @@ nsXMLHttpRequest::GetCurrentHttpChannel()
   return httpChannel;
 }
 
+inline PRBool
+IsSystemPrincipal(nsIPrincipal* aPrincipal)
+{
+  nsCOMPtr<nsIPrincipal> systemPrincipal;
+  nsresult rv = nsContentUtils::GetSecurityManager()->
+                  GetSystemPrincipal(getter_AddRefs(systemPrincipal));
+  if (NS_FAILED(rv))
+    return PR_FALSE;
+
+  return (aPrincipal == systemPrincipal);
+}
+
 
 /* noscript void openRequest (in AUTF8String method, in AUTF8String url, in boolean async, in AString user, in AString password); */
 NS_IMETHODIMP
@@ -1264,10 +1280,29 @@ nsXMLHttpRequest::OnDataAvailable(nsIRequest *request, nsISupports *ctxt, nsIInp
   return inStr->ReadSegments(nsXMLHttpRequest::StreamReaderFunc, (void*)this, count, &totalRead);
 }
 
+PRBool
+IsSameOrBaseChannel(nsIRequest* aPossibleBase, nsIChannel* aChannel)
+{
+  nsCOMPtr<nsIMultiPartChannel> mpChannel = do_QueryInterface(aPossibleBase);
+  if (mpChannel) {
+    nsCOMPtr<nsIChannel> baseChannel;
+    nsresult rv = mpChannel->GetBaseChannel(getter_AddRefs(baseChannel));
+    NS_ENSURE_SUCCESS(rv, PR_FALSE);
+    
+    return baseChannel == aChannel;
+  }
+
+  return aPossibleBase == aChannel;
+}
+
 /* void onStartRequest (in nsIRequest request, in nsISupports ctxt); */
 NS_IMETHODIMP
 nsXMLHttpRequest::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
+  if (!IsSameOrBaseChannel(request, mChannel)) {
+    return NS_OK;
+  }
+
   // Don't do anything if we have been aborted
   if (mState & XML_HTTP_REQUEST_UNINITIALIZED)
     return NS_OK;
@@ -1281,6 +1316,8 @@ nsXMLHttpRequest::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
   nsCOMPtr<nsIChannel> channel(do_QueryInterface(request));
   NS_ENSURE_TRUE(channel, NS_ERROR_UNEXPECTED);
 
+  channel->SetOwner(mPrincipal);
+
   mReadRequest = request;
   mContext = ctxt;
   mState |= XML_HTTP_REQUEST_PARSEBODY;
@@ -1315,6 +1352,9 @@ nsXMLHttpRequest::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
                                       getter_AddRefs(mDocument));
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
+  nsCOMPtr<nsIDocument> doc(do_QueryInterface(mDocument));
+  doc->SetPrincipal(mPrincipal);
+
   // Reset responseBody
   mResponseBody.Truncate();
 
@@ -1381,6 +1421,10 @@ nsXMLHttpRequest::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 NS_IMETHODIMP
 nsXMLHttpRequest::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult status)
 {
+  if (!IsSameOrBaseChannel(request, mChannel)) {
+    return NS_OK;
+  }
+
   // Don't do anything if we have been aborted
   if (mState & XML_HTTP_REQUEST_UNINITIALIZED)
     return NS_OK;
@@ -1535,7 +1579,7 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
   if (httpChannel) {
     httpChannel->GetRequestMethod(method); // If GET, method name will be uppercase
 
-    if (mPrincipal) {
+    if (mPrincipal && !IsSystemPrincipal(mPrincipal)) {
       nsCOMPtr<nsIURI> codebase;
       mPrincipal->GetURI(getter_AddRefs(codebase));
 
@@ -1941,7 +1985,8 @@ nsXMLHttpRequest::ChangeState(PRUint32 aState, PRBool aBroadcast,
   if ((mState & XML_HTTP_REQUEST_ASYNC) &&
       (aState & XML_HTTP_REQUEST_LOADSTATES) && // Broadcast load states only
       aBroadcast &&
-      onReadyStateChangeListener) {
+      onReadyStateChangeListener &&
+      NS_SUCCEEDED(CheckInnerWindowCorrectness())) {
     nsCOMPtr<nsIJSContextStack> stack;
     JSContext *cx = nsnull;
 
@@ -1977,39 +2022,26 @@ nsXMLHttpRequest::OnChannelRedirect(nsIChannel *aOldChannel,
 {
   NS_PRECONDITION(aNewChannel, "Redirect without a channel?");
 
-  if (mScriptContext && !(mState & XML_HTTP_REQUEST_XSITEENABLED)) {
+  if (!(mState & XML_HTTP_REQUEST_XSITEENABLED)) {
     nsresult rv = NS_ERROR_FAILURE;
 
-    nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1", & rv));
-    if (NS_FAILED(rv))
-      return rv;
-
-    JSContext *cx = (JSContext *)mScriptContext->GetNativeContext();
-    if (!cx)
-      return NS_ERROR_UNEXPECTED;
-
-    nsCOMPtr<nsIScriptSecurityManager> secMan =
-             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv))
-      return rv;
+    nsCOMPtr<nsIURI> oldURI;
+    rv = aOldChannel->GetURI(getter_AddRefs(oldURI));
+    NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIURI> newURI;
     rv = aNewChannel->GetURI(getter_AddRefs(newURI)); // The redirected URI
     if (NS_FAILED(rv))
       return rv;
 
-    stack->Push(cx);
+    nsCOMPtr<nsIScriptSecurityManager> secMan =
+             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+    if (NS_FAILED(rv))
+      return rv;
 
-    rv = secMan->CheckSameOrigin(cx, newURI);
-
-    stack->Pop(&cx);
-
-    if (NS_FAILED(rv)) {
-      // The security manager set a pending exception.  Since we're
-      // running under the event loop, we need to report it.
-      ::JS_ReportPendingException(cx);
+    rv = secMan->CheckSameOriginURI(oldURI, newURI);
+    if (NS_FAILED(rv))
       return rv;
-    }
   }
 
   if (mChannelEventSink) {
