@@ -47,7 +47,7 @@
 #include "nsForwardReference.h"
 #include "nsIContent.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIDOMXULCommandDispatcher.h"
+#include "nsXULCommandDispatcher.h"
 #include "nsIDOMXULDocument.h"
 #include "nsISupportsArray.h"
 #include "nsCOMArray.h"
@@ -84,6 +84,7 @@ struct PRLogModuleInfo;
 class nsXULDocument : public nsXMLDocument,
                       public nsIXULDocument,
                       public nsIDOMXULDocument2,
+                      public nsIDOMXULDocument_MOZILLA_1_8_BRANCH,
                       public nsIStreamLoaderObserver
 {
 public:
@@ -163,6 +164,7 @@ public:
     // nsIDOMXULDocument interface
     NS_DECL_NSIDOMXULDOCUMENT
     NS_DECL_NSIDOMXULDOCUMENT2
+    NS_DECL_NSIDOMXULDOCUMENT_MOZILLA_1_8_BRANCH
 
     // nsIDOMNSDocument
     NS_IMETHOD GetContentType(nsAString& aContentType);
@@ -272,7 +274,7 @@ protected:
     PRPackedBool               mApplyingPersistedAttrs;
     PRPackedBool               mIsWritingFastLoad;
     PRPackedBool               mDocumentLoaded;
-    nsCOMPtr<nsIDOMXULCommandDispatcher>     mCommandDispatcher; // [OWNER] of the focus tracker
+    nsRefPtr<nsXULCommandDispatcher> mCommandDispatcher; // [OWNER] of the focus tracker
 
     // Maintains the template builders that have been attached to
     // content elements
