@@ -198,11 +198,6 @@ JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                 jsval *rval);
 
-static JSFunctionSpec sXPC_NW_JSClass_methods[] = {
-  {"toString", XPC_NW_toString, 0, 0, 0},
-  {0, 0, 0, 0, 0}
-};
-
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPCNativeWrapperCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                      jsval *rval);
@@ -228,6 +223,61 @@ GetStringByIndex(JSContext *cx, uintN index)
   return ID_TO_VALUE(rt->GetStringID(index));
 }
 
+static inline
+already_AddRefed<nsIScriptSecurityManager>
+GetSecurityManager(JSContext *cx)
+{
+  XPCCallContext ccx(JS_CALLER, cx);
+  nsIXPCSecurityManager *sm = ccx.GetXPCContext()->
+    GetAppropriateSecurityManager(nsIXPCSecurityManager::HOOK_CALL_METHOD);
+  nsCOMPtr<nsIScriptSecurityManager> ssm(do_QueryInterface(sm));
+
+  nsIScriptSecurityManager *rval = nsnull;
+  ssm.swap(rval);
+  return rval;
+}
+
+static inline
+JSBool
+EnsureLegalActivity(JSContext *cx, JSObject *obj)
+{
+  jsval flags;
+
+  ::JS_GetReservedSlot(cx, obj, 0, &flags);
+  if (HAS_FLAGS(flags, FLAG_EXPLICIT)) {
+    // Can't make any assertions about the owner of this wrapper.
+    return JS_TRUE;
+  }
+
+  JSStackFrame *frame = nsnull;
+  uint32 fileFlags = JS_GetTopScriptFilenameFlags(cx, NULL);
+  if (!JS_FrameIterator(cx, &frame) ||
+      fileFlags == JSFILENAME_NULL ||
+      (fileFlags & JSFILENAME_SYSTEM)) {
+    // We expect implicit native wrappers in system files.
+    return JS_TRUE;
+  }
+
+  nsCOMPtr<nsIScriptSecurityManager> ssm(GetSecurityManager(cx));
+  if (!ssm) {
+    // If there's no security manager, then we're not running in a browser
+    // context: allow access.
+    return JS_TRUE;
+  }
+
+  // A last ditch effort to allow access: if the currently-running code
+  // has UniversalXPConnect privileges, then allow access.
+  PRBool isPrivileged;
+  nsresult rv = ssm->IsCapabilityEnabled("UniversalXPConnect", &isPrivileged);
+  if (NS_SUCCEEDED(rv) && isPrivileged) {
+    return JS_TRUE;
+  }
+
+  // Otherwise, we're looking at a non-system file with a handle on an
+  // implcit wrapper. This is a bug! Deny access.
+  return ThrowException(NS_ERROR_XPC_SECURITY_MANAGER_VETO, cx);
+}
+
 static JSBool
 WrapFunction(JSContext* cx, JSObject* funobj, jsval *rval)
 {
@@ -238,24 +288,56 @@ WrapFunction(JSContext* cx, JSObject* funobj, jsval *rval)
     *rval = OBJECT_TO_JSVAL(funobj);
     return JS_TRUE;
   }
+
+  // Ensure that we've been called from JS. Native code should extract
+  // the wrapped native and deal with that directly.
+  // XXX Can we simply trust |cx| here?
+  JSStackFrame *iterator = nsnull;
+  if (!::JS_FrameIterator(cx, &iterator)) {
+    ::JS_ReportError(cx, "XPCNativeWrappers must be used from script");
+    return nsnull;
+  }
   
   // Create a new function that'll call our given function.  This new
   // function's parent will be the original function and that's how we
   // get the right thing to call when this function is called.
+  // Note that we pass nsnull as the nominal parent so that we'll inherit
+  // our caller's Function.prototype.
   JSFunction *funWrapper =
-    ::JS_NewFunction(cx, XPC_NW_FunctionWrapper, 0, 0, funobj,
+    ::JS_NewFunction(cx, XPC_NW_FunctionWrapper, 0, 0, nsnull,
                      "XPCNativeWrapper function wrapper");
   if (!funWrapper) {
     return JS_FALSE;
   }
 
-  *rval = OBJECT_TO_JSVAL(::JS_GetFunctionObject(funWrapper));
+  JSObject* funWrapperObj = ::JS_GetFunctionObject(funWrapper);
+  ::JS_SetParent(cx, funWrapperObj, funobj);
+  *rval = OBJECT_TO_JSVAL(funWrapperObj);
   return JS_TRUE;
 }
 
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 {
+  JSProperty *prop;
+  JSObject *objp;
+  jsid idAsId;
+
+  if (!::JS_ValueToId(cx, id, &idAsId) ||
+      !OBJ_LOOKUP_PROPERTY(cx, obj, idAsId, &objp, &prop)) {
+    return JS_FALSE;
+  }
+
+  // Do not allow scripted getters or setters on XPCNativeWrappers.
+  NS_ASSERTION(prop && objp == obj, "Wasn't this property just added?");
+  JSScopeProperty *sprop = (JSScopeProperty *) prop;
+  if (sprop->attrs & (JSPROP_GETTER | JSPROP_SETTER)) {
+    OBJ_DROP_PROPERTY(cx, objp, prop);
+    return ThrowException(NS_ERROR_ILLEGAL_VALUE, cx);
+  }
+
+  OBJ_DROP_PROPERTY(cx, objp, prop);
+
   jsval flags;
   ::JS_GetReservedSlot(cx, obj, 0, &flags);
   if (!HAS_FLAGS(flags, FLAG_RESOLVING)) {
@@ -264,12 +346,17 @@ XPC_NW_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
   // Note: no need to protect *vp from GC here, since it's already in the slot
   // on |obj|.
-  return RewrapIfDeepWrapper(cx, obj, *vp, vp);
+  return EnsureLegalActivity(cx, obj) &&
+         RewrapIfDeepWrapper(cx, obj, *vp, vp);
 }
 
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 {
+  if (!EnsureLegalActivity(cx, obj)) {
+    return JS_FALSE;
+  }
+
   XPC_NW_BYPASS_BASE(cx, obj,
     // We're being notified of a delete operation on id in this
     // XPCNativeWrapper, so forward to the right high-level hook,
@@ -341,7 +428,8 @@ RewrapIfDeepWrapper(JSContext *cx, JSObject *obj, jsval v, jsval *rval)
     // Just using GetNewOrUsed on the return value of
     // GetWrappedNativeOfJSObject will give the right thing -- the unique deep
     // implicit wrapper associated with wrappedNative.
-    JSObject* wrapperObj = XPCNativeWrapper::GetNewOrUsed(cx, wrappedNative);
+    JSObject* wrapperObj = XPCNativeWrapper::GetNewOrUsed(cx, wrappedNative,
+                                                          nsnull);
     if (!wrapperObj) {
       return JS_FALSE;
     }
@@ -413,6 +501,10 @@ XPC_NW_GetOrSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp,
     }
   }
 
+  if (!EnsureLegalActivity(cx, obj)) {
+    return JS_FALSE;
+  }
+
   XPCWrappedNative *wrappedNative =
     XPCNativeWrapper::GetWrappedNative(cx, obj);
 
@@ -622,6 +714,10 @@ XPC_NW_Enumerate(JSContext *cx, JSObject *obj)
   // JS_Enumerate API.  Then reflect properties named by the enumerated
   // identifiers from the wrapped native to the native wrapper.
 
+  if (!EnsureLegalActivity(cx, obj)) {
+    return JS_FALSE;
+  }
+
   XPCWrappedNative *wn = XPCNativeWrapper::GetWrappedNative(cx, obj);
   if (!wn) {
     return JS_TRUE;
@@ -676,11 +772,20 @@ XPC_NW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
   // couldn't get at those values anyway.  Also, we always deal with
   // wrappedJSObject and toString before looking at our scriptable hooks, so no
   // need to mess with our flags yet.
-  if (id == GetStringByIndex(cx, XPCJSRuntime::IDX_WRAPPED_JSOBJECT) ||
-      id == GetStringByIndex(cx, XPCJSRuntime::IDX_TO_STRING)) {
+  if (id == GetStringByIndex(cx, XPCJSRuntime::IDX_WRAPPED_JSOBJECT)) {
     return JS_TRUE;
   }
 
+  if (!EnsureLegalActivity(cx, obj)) {
+    return JS_FALSE;
+  }
+
+  if (id == GetStringByIndex(cx, XPCJSRuntime::IDX_TO_STRING)) {
+    *objp = obj;
+    return JS_DefineFunction(cx, obj, "toString",
+                             XPC_NW_toString, 0, 0) != nsnull;
+  }
+
   // We can't use XPC_NW_BYPASS here, because we need to do a full
   // OBJ_LOOKUP_PROPERTY on the wrapped native's object, in order to
   // trigger reflection along the wrapped native prototype chain.
@@ -890,8 +995,11 @@ XPC_NW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
 JS_STATIC_DLL_CALLBACK(JSBool)
 XPC_NW_Convert(JSContext *cx, JSObject *obj, JSType type, jsval *vp)
 {
-  XPC_NW_BYPASS(cx, obj, convert, (cx, obj, type, vp));
+  if (!EnsureLegalActivity(cx, obj)) {
+    return JS_FALSE;
+  }
 
+  XPC_NW_BYPASS(cx, obj, convert, (cx, obj, type, vp));
   return JS_TRUE;
 }
 
@@ -1006,7 +1114,7 @@ MirrorWrappedNativeParent(JSContext *cx, XPCWrappedNative *wrapper,
     XPCWrappedNative *parent_wrapper =
       XPCWrappedNative::GetWrappedNativeOfJSObject(cx, wn_parent);
 
-    *result = XPCNativeWrapper::GetNewOrUsed(cx, parent_wrapper);
+    *result = XPCNativeWrapper::GetNewOrUsed(cx, parent_wrapper, nsnull);
     if (!*result)
       return JS_FALSE;
   }
@@ -1024,7 +1132,7 @@ XPCNativeWrapperCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
   // |obj| almost always has the wrong proto and parent so we have to create
   // our own object anyway.  Set |obj| to null so we don't use it by accident.
   obj = nsnull;
-  
+
   jsval native = argv[0];
 
   if (JSVAL_IS_PRIMITIVE(native)) {
@@ -1077,9 +1185,12 @@ XPCNativeWrapperCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
   printf("Creating new JSObject\n");
 #endif
   wrapperObj = ::JS_NewObject(cx, XPCNativeWrapper::GetJSClass(), nsnull,
-                              wrappedNative->GetScope()->GetGlobalJSObject());
+                              nsnull);
 
-  if (!wrapperObj) {
+  if (!wrapperObj ||
+      !::JS_SetParent(cx, wrapperObj,
+                      wrappedNative->GetScope()->GetGlobalJSObject()) ||
+      !::JS_SetPrototype(cx, wrapperObj, nsnull)) {
     // JS_NewObject already threw (or reported OOM).
     return JS_FALSE;
   }
@@ -1245,6 +1356,10 @@ XPC_NW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     }
   }
 
+  if (!EnsureLegalActivity(cx, obj)) {
+    return JS_FALSE;
+  }
+
   // Check whether toString was overridden in any object along
   // the wrapped native's object's prototype chain.
   XPCJSRuntime *rt = nsXPConnect::GetRuntime();
@@ -1357,7 +1472,7 @@ XPCNativeWrapper::AttachNewConstructorObject(XPCCallContext &ccx,
 {
   JSObject *class_obj =
     ::JS_InitClass(ccx, aGlobalObject, nsnull, &sXPC_NW_JSClass.base,
-                   XPCNativeWrapperCtor, 0, nsnull, sXPC_NW_JSClass_methods,
+                   XPCNativeWrapperCtor, 0, nsnull, nsnull,
                    nsnull, nsnull);
   if (!class_obj) {
     NS_WARNING("can't initialize the XPCNativeWrapper class");
@@ -1381,8 +1496,28 @@ XPCNativeWrapper::AttachNewConstructorObject(XPCCallContext &ccx,
 
 // static
 JSObject *
-XPCNativeWrapper::GetNewOrUsed(JSContext *cx, XPCWrappedNative *wrapper)
+XPCNativeWrapper::GetNewOrUsed(JSContext *cx, XPCWrappedNative *wrapper,
+                               JSObject *callee)
 {
+  if (callee) {
+    nsCOMPtr<nsIPrincipal> prin;
+
+    nsCOMPtr<nsIScriptSecurityManager> ssm = GetSecurityManager(cx);
+    nsresult rv = ssm
+                  ? ssm->GetObjectPrincipal(cx, callee, getter_AddRefs(prin))
+                  : NS_ERROR_FAILURE;
+    if (NS_SUCCEEDED(rv) && prin) {
+      nsCOMPtr<nsIPrincipal> sysprin;
+      rv = ssm->GetSystemPrincipal(getter_AddRefs(sysprin));
+      if (NS_SUCCEEDED(rv) && sysprin != prin) {
+        jsval v = OBJECT_TO_JSVAL(wrapper->GetFlatJSObject());
+        if (!XPCNativeWrapperCtor(cx, JSVAL_TO_OBJECT(v), 1, &v, &v))
+          return nsnull;
+        return JSVAL_TO_OBJECT(v);
+      }
+    }
+  }
+
   // Prevent wrapping a double-wrapped JS object in an
   // XPCNativeWrapper!
   nsCOMPtr<nsIXPConnectWrappedJS> xpcwrappedjs(do_QueryWrappedNative(wrapper));
@@ -1419,14 +1554,16 @@ XPCNativeWrapper::GetNewOrUsed(JSContext *cx, XPCWrappedNative *wrapper)
     ::JS_LockGCThing(cx, nw_parent);
   }
 
-  obj = ::JS_NewObject(cx, GetJSClass(), nsnull, nw_parent);
+  obj = ::JS_NewObject(cx, GetJSClass(), nsnull, nsnull);
 
   if (lock) {
     ::JS_UnlockGCThing(cx, nw_parent);
   }
 
   if (!obj ||
+      !::JS_SetParent(cx, obj, nw_parent) ||
       !::JS_SetPrivate(cx, obj, wrapper) ||
+      !::JS_SetPrototype(cx, obj, nsnull) ||
       !::JS_SetReservedSlot(cx, obj, 0, INT_TO_JSVAL(FLAG_DEEP))) {
     return nsnull;
   }
