@@ -320,8 +320,10 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
     fm->GetMaxDescent(aDesiredSize.descent);
   }
   while (childFrame) {
+    nsReflowReason reason = (childFrame->GetStateBits() & NS_FRAME_FIRST_REFLOW)
+      ? eReflowReason_Initial : aReflowState.reason;
     nsHTMLReflowState childReflowState(aPresContext, aReflowState,
-                                       childFrame, availSize);
+                                       childFrame, availSize, reason);
     rv = mathMLFrame->ReflowChild(childFrame, aPresContext, childDesiredSize,
                                   childReflowState, childStatus);
     //NS_ASSERTION(NS_FRAME_IS_COMPLETE(childStatus), "bad status");
@@ -479,6 +481,9 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
   mathMLFrame->SetBoundingMetrics(aDesiredSize.mBoundingMetrics);
   mathMLFrame->SetReference(nsPoint(0, aDesiredSize.ascent));
 
+  // see if we should fix the spacing
+  mathMLFrame->FixInterFrameSpacing(aDesiredSize);
+
   aStatus = NS_FRAME_COMPLETE;
   NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
@@ -589,6 +594,31 @@ nsMathMLmfencedFrame::PlaceChar(nsMathMLChar*      aMathMLChar,
   dx += rect.width;
 }
 
+nscoord
+nsMathMLmfencedFrame::FixInterFrameSpacing(nsHTMLReflowMetrics& aDesiredSize)
+{
+  nscoord gap = nsMathMLContainerFrame::FixInterFrameSpacing(aDesiredSize);
+  if (!gap) return 0;
+
+  nsRect rect;
+  if (mOpenChar) {
+    mOpenChar->GetRect(rect);
+    rect.MoveBy(gap, 0);
+    mOpenChar->SetRect(rect);
+  }
+  if (mCloseChar) {
+    mCloseChar->GetRect(rect);
+    rect.MoveBy(gap, 0);
+    mCloseChar->SetRect(rect);
+  }
+  for (PRInt32 i = 0; i < mSeparatorsCount; i++) {
+    mSeparatorsChar[i].GetRect(rect);
+    rect.MoveBy(gap, 0);
+    mSeparatorsChar[i].SetRect(rect);
+  }
+  return gap;
+}
+
 // ----------------------
 // the Style System will use these to pass the proper style context to our MathMLChar
 nsStyleContext*
