@@ -22,7 +22,7 @@
  *
  * Contributor(s):
  *   Jan Varga (varga@ku.sk)
- *   Hkan Waara (hwaara@chello.se)
+ *   Håkan Waara (hwaara@chello.se)
  *   Neil Rashbrook (neil@parkwaycc.co.uk)
  *   Seth Spitzer <sspitzer@netscape.com>
  *
@@ -110,6 +110,47 @@ function SelectAndScrollToKey(aMsgKey)
   return true;
 }
 
+// A helper routine called after a folder is loaded to make sure
+// we select and scroll to the correct message (could be the first new message,
+// could be the last displayed message, etc.)
+// returns true if we ended up scrolling to a message
+function ScrollToMessageAfterFolderLoad(folder)
+{
+  var scrolled = pref.getBoolPref("mailnews.scroll_to_new_message") &&
+                 ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
+  if (!scrolled && folder && pref.getBoolPref("mailnews.remember_selected_message"))
+  {
+    // reselect the last selected message
+    var lastMessageLoaded = folder.lastMessageLoaded;
+    if (lastMessageLoaded != nsMsgKey_None)
+      scrolled = SelectAndScrollToKey(lastMessageLoaded);
+  }
+
+  if (!scrolled) 
+  {
+    // if we still haven't scrolled,
+    // scroll to the newest, which might be the top or the bottom
+    // depending on our sort order and sort type
+    if (gDBView.sortOrder == nsMsgViewSortOrder.ascending) 
+    {
+      switch (gDBView.sortType) 
+      {
+        case nsMsgViewSortType.byDate: 
+        case nsMsgViewSortType.byId: 
+        case nsMsgViewSortType.byThread: 
+         scrolled = ScrollToMessage(nsMsgNavigationType.lastMessage, true, false /* selectMessage */);
+         break;
+      }
+    }
+
+    // if still we haven't scrolled,
+    // scroll to the top.
+    if (!scrolled)
+      EnsureRowInThreadTreeIsVisible(0);
+  }
+  return scrolled;
+}
+
 // the folderListener object
 var folderListener = {
     OnItemAdded: function(parentItem, item) { },
@@ -136,6 +177,7 @@ var folderListener = {
        var eventType = event.toString();
        if (eventType == "FolderLoaded") {
          if (folder) {
+           var scrolled = false;  
            var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
            var uri = folder.URI;
            var rerootingFolder = (uri == gCurrentFolderToReroot);
@@ -168,9 +210,7 @@ var folderListener = {
                gCurrentLoadingFolderViewType = 0;
                gCurrentLoadingFolderViewFlags = 0;
 
-               var scrolled = false;
-
-               LoadCurrentlyDisplayedMessage();  //used for rename folder msg loading after folder is loaded.
+               scrolled = LoadCurrentlyDisplayedMessage();  //used for rename folder msg loading after folder is loaded.
 
                if (gStartMsgKey != nsMsgKey_None) {
                  scrolled = SelectAndScrollToKey(gStartMsgKey);
@@ -187,53 +227,10 @@ var folderListener = {
              }
            }
            if (uri == gCurrentLoadingFolderURI) {
-             // NOTE,
-             // if you change the scrolling code below,
-             // double check the scrolling logic in
-             // searchBar.js, restorePreSearchView()
              viewDebug("uri == current loading folder uri\n");
              gCurrentLoadingFolderURI = "";
-
-             // if we didn't just scroll, 
-             // scroll to the first new message
-             // but don't select it
-             if (!scrolled && pref.getBoolPref("mailnews.scroll_to_new_message"))
-               scrolled = ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
-
-             if (!scrolled && pref.getBoolPref("mailnews.remember_selected_message")) {
-               // if we failed to scroll to a new message,
-               // reselect the last selected message
-               var lastMessageLoaded = msgFolder.lastMessageLoaded;
-               if (lastMessageLoaded != nsMsgKey_None) {
-                 scrolled = SelectAndScrollToKey(lastMessageLoaded);
-               }
-             }
-
-             if (!scrolled) {
-               // if we still haven't scrolled,
-               // scroll to the newest, which might be the top or the bottom
-               // depending on our sort order and sort type
-               if (gDBView.sortOrder == nsMsgViewSortOrder.ascending) {
-                 switch (gDBView.sortType) {
-                   case nsMsgViewSortType.byDate: 
-                   case nsMsgViewSortType.byId: 
-                   case nsMsgViewSortType.byThread: 
-                     scrolled = ScrollToMessage(nsMsgNavigationType.lastMessage, true, false /* selectMessage */);
-                     break;
-                 }
-               }
-
-               // if still we haven't scrolled,
-               // scroll to the top.
-               if (!scrolled)
-                 EnsureRowInThreadTreeIsVisible(0);
-             }            
-
-             // NOTE,
-             // if you change the scrolling code above,
-             // double check the scrolling logic in
-             // searchBar.js, restorePreSearchView()
-
+             if (!scrolled)
+               ScrollToMessageAfterFolderLoad(msgFolder);
              SetBusyCursor(window, false);
            }
            if (gNotifyDefaultInboxLoadedOnStartup && (folder.flags & 0x1000))
@@ -261,7 +258,8 @@ var folderListener = {
              {
                 gDefaultSearchViewTerms = null;
                 viewDebug("searching gVirtualFolderTerms\n");
-                loadVirtualFolder();
+                gDBView.viewFolder = gMsgFolderSelected;
+                ViewChangeByFolder(gMsgFolderSelected);
              }
              else if (gMsgFolderSelected.flags & MSG_FOLDER_FLAG_VIRTUAL)
              {
@@ -273,13 +271,10 @@ var folderListener = {
                // get the view value from the folder
                if (msgFolder)
                {
-                 var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
-                 var dbFolderInfo = msgDatabase.dBFolderInfo; 
-                 var result = dbFolderInfo.getUint32Property("current-view", 0);
-                 
                  // if our new view is the same as the old view and we already have the list of search terms built up
                  // for the old view, just re-use it
-                 if (gCurrentViewValue == result && gDefaultSearchViewTerms)
+                 var result = GetMailViewForFolder(msgFolder);
+                 if (GetSearchInput() && gCurrentViewValue == result && gDefaultSearchViewTerms)
                  {
                    viewDebug("searching gDefaultSearchViewTerms and rerootingFolder\n");
                    Search("");
@@ -344,8 +339,29 @@ var folderObserver = {
         DropOnFolderTree(row, orientation);
     },
 
-    onToggleOpenState: function()
+    onToggleOpenState: function(index)
     {
+      var folderTree = GetFolderTree();
+
+      // Nothing to do when collapsing an item.
+      if (folderTree.view.isContainerOpen(index))
+        return;
+
+      var folderResource = GetFolderResource(folderTree, index);
+
+      if (folderTree.view.getLevel(index) == 0)
+      {
+        // (Imap/Nntp/Pop) Account item.
+
+        folderResource.QueryInterface(Components.interfaces.nsIMsgFolder)
+                      .server.performExpand(msgWindow);
+      }
+      else if (folderResource instanceof Components.interfaces.nsIMsgImapMailFolder)
+      {
+        // Imap message folder item.
+
+        folderResource.performExpand(msgWindow);
+      }
     },
 
     onCycleHeader: function(colID, elt)
@@ -547,7 +563,8 @@ function HandleCompactCompleted(folder)
 
 function LoadCurrentlyDisplayedMessage()
 {
-  if (gCurrentlyDisplayedMessage != nsMsgViewIndex_None)
+  var scrolled = (gCurrentlyDisplayedMessage != nsMsgViewIndex_None);
+  if (scrolled)
   {
     var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
     var treeSelection = treeView.selection;
@@ -558,6 +575,7 @@ function LoadCurrentlyDisplayedMessage()
     SetFocusThreadPane();
     gCurrentlyDisplayedMessage = nsMsgViewIndex_None; //reset
   }
+  return scrolled;
 }
 
 function IsCurrentLoadedFolder(folder)
@@ -693,12 +711,13 @@ function OnLoadMessenger()
   ShowHideToolBarButtons();
   verifyAccounts(null);
     
-  HideAccountCentral();
   InitMsgWindow();
   messenger.SetWindow(window, msgWindow);
 
   InitializeDataSources();
   InitPanes();
+  
+  MigrateJunkMailSettings();
 
   accountManager.setSpecialFolders();
   accountManager.loadVirtualFolders();
@@ -755,6 +774,10 @@ function NotifyObservers(aSubject, aTopic, aData)
 
 function Create3PaneGlobals()
 {
+  accountCentralBox = document.getElementById("accountCentralBox");
+  gSearchBox = document.getElementById("searchBox");
+  gSearchBox.collapsed = true;
+  GetMessagePane().collapsed = true;
 }
  
 // because the "open" state persists, we'll call
@@ -826,8 +849,8 @@ function loadStartFolder(initialUri)
         // Perform biff on the server to check for new mail, except for 
         // a pop3 account that is deferred or deferred to,
         // or the case where initialUri is non-null (non-startup)
-        if (!initialUri && isLoginAtStartUpEnabled && gLoadStartFolder
-            && !defaultServer.isDeferredTo &&
+        if (!initialUri && isLoginAtStartUpEnabled && gLoadStartFolder &&
+            !gNewAccountToLoad && !defaultServer.isDeferredTo &&
             defaultServer.rootFolder == defaultServer.rootMsgFolder)
           defaultServer.PerformBiff(msgWindow);        
 
@@ -857,7 +880,8 @@ function loadStartFolder(initialUri)
             InitPrompts();
             InitServices();
 
-            if (gPromptService) 
+            var sendUnsentWhenGoingOnlinePref = pref.getIntPref("offline.send.unsent_messages");
+            if(gPromptService && sendUnsentWhenGoingOnlinePref == 0) // pref is "ask"
             {
                 var buttonPressed = gPromptService.confirmEx(window, 
                                     gOfflinePromptsBundle.getString('sendMessagesOfflineWindowTitle'), 
@@ -867,9 +891,11 @@ function loadStartFolder(initialUri)
                                     gOfflinePromptsBundle.getString('sendMessagesSendButtonLabel'),
                                     gOfflinePromptsBundle.getString('sendMessagesNoSendButtonLabel'),
                                     null, null, {value:0});
-                if (buttonPressed == 0) 
+                if (buttonPressed == 0)
                     SendUnsentMessages();
             }
+            else if(sendUnsentWhenGoingOnlinePref == 1) // pref is "yes"
+                SendUnsentMessages();
         }
     }
 }
@@ -882,7 +908,7 @@ function AddToSession()
         var nsIFolderListener = Components.interfaces.nsIFolderListener;
         var notifyFlags = nsIFolderListener.intPropertyChanged | nsIFolderListener.event;
         mailSession.AddFolderListener(folderListener, notifyFlags);
-	} catch (ex) {
+    } catch (ex) {
         dump("Error adding to session\n");
     }
 }
@@ -1231,7 +1257,7 @@ function ChangeSelectionWithoutContentLoad(event, tree)
         if(tree.id == "threadTree")
           gThreadPaneCurrentSelectedIndex = row;
     }
-    event.preventBubble();
+    event.stopPropagation();
 }
 
 function TreeOnMouseDown(event)
@@ -1262,61 +1288,23 @@ function FolderPaneOnClick(event)
     if (row.value == -1) {
       if (event.originalTarget.localName == "treecol")
         // clicking on the name column in the folder pane should not sort
-        event.preventBubble();
-      return;
-    }
-
-    if (elt.value == "twisty")
-    {
-        if (!(folderTree.treeBoxObject.view.isContainerOpen(row.value)))
-        {
-            var folderResource = GetFolderResource(folderTree, row.value);
-
-            var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
-            if (isServer == "true")
-            {
-                var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-                var server = msgFolder.server;
-                server.performExpand(msgWindow);
-            }
-            else
-            {
-                var serverType = GetFolderAttribute(folderTree, folderResource, "ServerType");
-                if (serverType == "imap")
-                {
-                    var imapFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgImapMailFolder);
-                    imapFolder.performExpand(msgWindow);
-                }
-            }
-        }
+        event.stopPropagation();
     }
     else if ((event.originalTarget.localName == "slider") ||
              (event.originalTarget.localName == "scrollbarbutton")) {
-      event.preventBubble();
+      event.stopPropagation();
     }
-    else if (event.detail == 2) {
+    else if ((event.detail == 2) && (elt.value != "twisty") &&
+             (folderTree.view.getLevel(row.value) != 0)) {
       FolderPaneDoubleClick(row.value, event);
     }
-
 }
 
 function FolderPaneDoubleClick(folderIndex, event)
 {
-    var folderTree = GetFolderTree();
-    var folderResource = GetFolderResource(folderTree, folderIndex);
-
-    var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
-    if (isServer == "true")
-    {
-      if (!(folderTree.treeBoxObject.view.isContainerOpen(folderIndex)))
-      {
-        var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-        var server = msgFolder.server;
-        server.performExpand(msgWindow);
-      }
-    }
-    else 
+    if (!pref.getBoolPref("mailnews.reuse_thread_window2"))
     {
+      var folderResource = GetFolderResource(GetFolderTree(), folderIndex);
       // Open a new msg window only if we are double clicking on 
       // folders or newsgroups.
       MsgOpenNewWindowForFolder(folderResource.Value, -1 /* key */);
@@ -1324,7 +1312,7 @@ function FolderPaneDoubleClick(folderIndex, event)
       // double clicking should not toggle the open / close state of the
       // folder.  this will happen if we don't prevent the event from
       // bubbling to the default handler in tree.xml
-      event.preventBubble();
+      event.stopPropagation();
     }
 }
 
@@ -1531,11 +1519,6 @@ function SelectMessage(messageUri)
     gDBView.selectMsgByKey(msgHdr.messageKey);
 }
 
-function ReloadWithAllParts()
-{
-  gDBView.reloadMessageWithAllParts();
-}
-
 function ReloadMessage()
 {
   gDBView.reloadMessage();
@@ -1583,3 +1566,34 @@ function GetFolderAttribute(tree, source, attribute)
     return target;
 }
 
+// Some of the per account junk mail settings have been
+// converted to global prefs. Let's try to migrate some
+// of those settings from the default account.
+function MigrateJunkMailSettings()
+{
+  var junkMailSettingsVersion = pref.getIntPref("mail.spam.version");
+  if (!junkMailSettingsVersion)
+  {
+    // get the default account, check to see if we have values for our 
+    // globally migrated prefs.
+    var defaultAccount;
+    try {
+      defaultAccount = accountManager.defaultAccount;
+    } catch (ex) {}
+    if (defaultAccount && defaultAccount.incomingServer)
+    {
+      // we only care about
+      var prefix = "mail.server." + defaultAccount.incomingServer.key + ".";
+      if (pref.prefHasUserValue(prefix + "manualMark"))
+        pref.setBoolPref("mail.spam.manualMark", pref.getBoolPref(prefix + "manualMark"));
+      if (pref.prefHasUserValue(prefix + "manualMarkMode"))
+        pref.setIntPref("mail.spam.manualMarkMode", pref.getIntPref(prefix + "manualMarkMode"));
+      if (pref.prefHasUserValue(prefix + "spamLoggingEnabled"))
+        pref.setBoolPref("mail.spam.logging.enabled", pref.getBoolPref(prefix + "spamLoggingEnabled"));
+      if (pref.prefHasUserValue(prefix + "markAsReadOnSpam"))
+        pref.setBoolPref("mail.spam.markAsReadOnSpam", pref.getBoolPref(prefix + "markAsReadOnSpam"));
+    }  
+    // bump the version so we don't bother doing this again.
+    pref.setIntPref("mail.spam.version", 1);
+  }
+}
