@@ -72,14 +72,15 @@
 PRLogModuleInfo *gImgLog = PR_NewLogModule("imgRequest");
 #endif
 
-NS_IMPL_THREADSAFE_ISUPPORTS6(imgRequest, imgILoad,
+NS_IMPL_THREADSAFE_ISUPPORTS8(imgRequest, imgILoad,
                               imgIDecoderObserver, imgIContainerObserver,
                               nsIStreamListener, nsIRequestObserver,
-                              nsISupportsWeakReference)
+                              nsISupportsWeakReference, nsIChannelEventSink,
+                              nsIInterfaceRequestor)
 
 imgRequest::imgRequest() : 
   mObservers(0),
-  mLoading(PR_FALSE), mProcessing(PR_FALSE),
+  mLoading(PR_FALSE), mProcessing(PR_FALSE), mHadLastPart(PR_FALSE),
   mImageStatus(imgIRequest::STATUS_NONE), mState(0),
   mCacheId(0), mValidator(nsnull), mIsMultiPartChannel(PR_FALSE)
 {
@@ -106,6 +107,7 @@ nsresult imgRequest::Init(nsIChannel *aChannel,
     return NS_ERROR_OUT_OF_MEMORY;
 
   mChannel = aChannel;
+  mChannel->SetNotificationCallbacks(this);
 
   /* set our loading flag to true here.
      Setting it here lets checks to see if the load is in progress
@@ -157,7 +159,7 @@ nsresult imgRequest::RemoveProxy(imgRequestProxy *proxy, nsresult aStatus, PRBoo
 
   // make sure that observer gets an OnStopRequest message sent to it
   if (!(mState & onStopRequest)) {
-    proxy->OnStopRequest(nsnull, nsnull, NS_BINDING_ABORTED);
+    proxy->OnStopRequest(nsnull, nsnull, NS_BINDING_ABORTED, PR_TRUE);
   }
 
   if (mImage && !HaveProxyWithObserver(nsnull)) {
@@ -187,7 +189,7 @@ nsresult imgRequest::RemoveProxy(imgRequestProxy *proxy, nsresult aStatus, PRBoo
   // If a proxy is removed for a reason other than its owner being
   // changed, remove the proxy from the loadgroup.
   if (aStatus != NS_IMAGELIB_CHANGING_OWNER)
-    proxy->RemoveFromLoadGroup();
+    proxy->RemoveFromLoadGroup(PR_TRUE);
 
   return NS_OK;
 }
@@ -196,6 +198,10 @@ nsresult imgRequest::NotifyProxyListener(imgRequestProxy *proxy)
 {
   nsCOMPtr<imgIRequest> kungFuDeathGrip(proxy);
 
+  // OnStartRequest
+  if (mState & onStartRequest)
+    proxy->OnStartRequest(nsnull, nsnull);
+
   // OnStartDecode
   if (mState & onStartDecode)
     proxy->OnStartDecode();
@@ -250,7 +256,9 @@ nsresult imgRequest::NotifyProxyListener(imgRequestProxy *proxy)
   }
 
   if (mState & onStopRequest) {
-    proxy->OnStopRequest(nsnull, nsnull, GetResultFromImageStatus(mImageStatus));
+    proxy->OnStopRequest(nsnull, nsnull,
+                         GetResultFromImageStatus(mImageStatus),
+                         mHadLastPart);
   }
 
   return NS_OK;
@@ -305,6 +313,23 @@ nsresult imgRequest::GetURI(nsIURI **aURI)
   return NS_ERROR_FAILURE;
 }
 
+nsresult
+imgRequest::GetCurrentURI(nsIURI **aURI)
+{
+  LOG_FUNC(gImgLog, "imgRequest::GetCurrentURI");
+
+  if (mChannel)
+    return mChannel->GetURI(aURI);
+
+  if (mCurrentURI) {
+    *aURI = mCurrentURI;
+    NS_ADDREF(*aURI);
+    return NS_OK;
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
 void imgRequest::RemoveFromCache()
 {
   LOG_SCOPE(gImgLog, "imgRequest::RemoveFromCache");
@@ -598,7 +623,6 @@ NS_IMETHODIMP imgRequest::OnStopDecode(imgIRequest *aRequest,
   return NS_OK;
 }
 
-
 /** nsIRequestObserver methods **/
 
 /* void onStartRequest (in nsIRequest request, in nsISupports ctxt); */
@@ -617,23 +641,21 @@ NS_IMETHODIMP imgRequest::OnStartRequest(nsIRequest *aRequest, nsISupports *ctxt
      Cancel() gets called, we have a channel to cancel and we don't leave the channel
      open forever.
    */
+  nsCOMPtr<nsIMultiPartChannel> mpchan(do_QueryInterface(aRequest));
   if (!mChannel) {
-    nsCOMPtr<nsIMultiPartChannel> mpchan(do_QueryInterface(aRequest));
     if (mpchan)
       mpchan->GetBaseChannel(getter_AddRefs(mChannel));
     else
       mChannel = do_QueryInterface(aRequest);
   }
 
-  nsCAutoString contentType;
-  mChannel->GetContentType(contentType);
-  if (contentType.Equals(NS_LITERAL_CSTRING("multipart/x-mixed-replace"),
-                         nsCaseInsensitiveCStringComparator()))
+  if (mpchan)
       mIsMultiPartChannel = PR_TRUE;
 
-  /* set our state variables to their initial values. */
+  /* set our state variables to their initial values, but advance mState
+     to onStartRequest. */
   mImageStatus = imgIRequest::STATUS_NONE;
-  mState = 0;
+  mState = onStartRequest;
 
   /* set our loading flag to true */
   mLoading = PR_TRUE;
@@ -725,8 +747,20 @@ NS_IMETHODIMP imgRequest::OnStopRequest(nsIRequest *aRequest, nsISupports *ctxt,
   /* set our processing flag to false */
   mProcessing = PR_FALSE;
 
+  mHadLastPart = PR_TRUE;
+  nsCOMPtr<nsIMultiPartChannel> mpchan(do_QueryInterface(aRequest));
+  if (mpchan) {
+    PRBool lastPart;
+    nsresult rv = mpchan->GetIsLastPart(&lastPart);
+    if (NS_SUCCEEDED(rv))
+      mHadLastPart = lastPart;
+  }
+
+  // XXXldb What if this is a non-last part of a multipart request?
   if (mChannel) {
     mChannel->GetOriginalURI(getter_AddRefs(mURI));
+    mChannel->GetURI(getter_AddRefs(mCurrentURI));
+    mChannel->SetNotificationCallbacks(nsnull);
     mChannel = nsnull; // we no longer need the channel
   }
 
@@ -756,7 +790,7 @@ NS_IMETHODIMP imgRequest::OnStopRequest(nsIRequest *aRequest, nsISupports *ctxt,
     /* calling OnStopRequest may result in the death of |proxy| so don't use the
        pointer after this call.
      */
-    if (proxy) proxy->OnStopRequest(aRequest, ctxt, status);
+    if (proxy) proxy->OnStopRequest(aRequest, ctxt, status, mHadLastPart);
   }
 
   return NS_OK;
@@ -918,3 +952,40 @@ imgRequest::SniffMimeType(const char *buf, PRUint32 len)
 {
   imgLoader::GetMimeTypeFromContent(buf, len, mContentType);
 }
+
+/** nsIInterfaceRequestor methods **/
+
+NS_IMETHODIMP
+imgRequest::GetInterface(const nsIID & aIID, void **aResult)
+{
+  return QueryInterface(aIID, aResult);
+}
+
+/** nsIChannelEventSink methods **/
+
+/* void onChannelRedirect (in nsIChannel oldChannel, in nsIChannel newChannel, in unsigned long flags); */
+NS_IMETHODIMP
+imgRequest::OnChannelRedirect(nsIChannel *oldChannel, nsIChannel *newChannel, PRUint32 flags)
+{
+  NS_ASSERTION(mChannel, "Got an OnChannelRedirect after we nulled out mChannel!");
+  NS_ASSERTION(mChannel == oldChannel, "Got a channel redirect for an unknown channel!");
+  NS_ASSERTION(newChannel, "Got a redirect to a NULL channel!");
+
+  RemoveFromCache();
+
+  nsCOMPtr<nsIURI> uri;
+  if (NS_SUCCEEDED(newChannel->GetURI(getter_AddRefs(uri))) && uri)
+    mCurrentURI = uri;
+
+  oldChannel->SetNotificationCallbacks(nsnull);
+  mChannel = newChannel;
+  mChannel->SetNotificationCallbacks(this);
+
+  // If we don't still have a cache entry, we don't want to refresh the cache.
+  if (uri && mCacheEntry) {
+    imgCache::Put(uri, this, getter_AddRefs(mCacheEntry));
+  }
+
+  return NS_OK;
+}
+
