@@ -245,11 +245,10 @@ public:
     return 0;
   }
 private:
-  static unsigned ThreadMain(void *p)
+  static void ThreadMain(void *p)
   {
     Thread *self = (Thread *) p;
     self->mThreadFunc(self->mThreadParam);
-    return 0;
   }
   int        mThread;
   ThreadFunc mThreadFunc;
@@ -264,6 +263,9 @@ private:
 
 static char* gSourcePath;
 static ArchiveReader gArchiveReader;
+#ifdef XP_WIN
+static bool gSucceeded = FALSE;
+#endif
 
 static const char kWhitespace[] = " \t";
 static const char kNL[] = "\r\n";
@@ -354,6 +356,33 @@ mstrtok(const char *delims, char **str)
   return ret;
 }
 
+static void ensure_write_permissions(const char *path)
+{
+#ifdef XP_WIN
+  (void)_chmod(path, _S_IREAD | _S_IWRITE);
+#else
+  struct stat fs;
+  if (!stat(path, &fs) && !(fs.st_mode & S_IWUSR)) {
+    (void)chmod(path, fs.st_mode | S_IWUSR);
+  }
+#endif
+}
+
+static int ensure_remove(const char *path)
+{
+  ensure_write_permissions(path);
+  int rv = remove(path);
+  if (rv)
+    LOG(("remove failed: %d,%d (%s)\n", rv, errno, path));
+  return rv;
+}
+
+static int ensure_open(const char *path, int flags, int options)
+{
+  ensure_write_permissions(path);
+  return open(path, flags, options);
+}
+
 // Ensure that the directory containing this file exists.
 static int ensure_parent_dir(const char *path)
 {
@@ -392,7 +421,7 @@ static int copy_file(const char *spath, const char *dpath)
     return READ_ERROR;
   }
 
-  AutoFD dfd = open(dpath, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, ss.st_mode);
+  AutoFD dfd = ensure_open(dpath, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, ss.st_mode);
   if (dfd < 0) {
     LOG(("copy_file: failed to open: %s,%d\n", dpath, errno));
     return WRITE_ERROR;
@@ -446,7 +475,7 @@ static int backup_restore(const char *path)
   if (rv)
     return rv;
 
-  rv = remove(backup);
+  rv = ensure_remove(backup);
   if (rv)
     return WRITE_ERROR;
 
@@ -459,7 +488,7 @@ static int backup_discard(const char *path)
   char backup[MAXPATHLEN];
   snprintf(backup, sizeof(backup), "%s" BACKUP_EXT, path);
 
-  int rv = remove(backup);
+  int rv = ensure_remove(backup);
   if (rv)
     return WRITE_ERROR;
 
@@ -592,11 +621,9 @@ RemoveFile::Execute()
     return rv;
   }
 
-  rv = remove(mFile);
-  if (rv) {
-    LOG(("remove failed: %d\n", rv));
+  rv = ensure_remove(mFile);
+  if (rv)
     return WRITE_ERROR;
-  }
 
   return OK;
 }
@@ -660,7 +687,7 @@ AddFile::Execute()
     if (rv)
       return rv;
 
-    rv = remove(mFile);
+    rv = ensure_remove(mFile);
     if (rv)
       return WRITE_ERROR;
   }
@@ -716,7 +743,7 @@ PatchFile::~PatchFile()
   // delete the temporary patch file
   char spath[MAXPATHLEN];
   snprintf(spath, MAXPATHLEN, "%s/%d.patch", gSourcePath, mPatchIndex);
-  remove(spath);
+  ensure_remove(spath);
 
   free(buf);
 }
@@ -794,7 +821,7 @@ PatchFile::Prepare()
   char spath[MAXPATHLEN];
   snprintf(spath, MAXPATHLEN, "%s/%d.patch", gSourcePath, mPatchIndex);
 
-  remove(spath);
+  ensure_remove(spath);
 
   int rv = gArchiveReader.ExtractFile(mPatchFile, spath);
   if (rv)
@@ -837,11 +864,11 @@ PatchFile::Execute()
   if (rv)
     return rv;
 
-  rv = remove(mFile);
+  rv = ensure_remove(mFile);
   if (rv)
     return WRITE_ERROR;
 
-  AutoFD ofd = open(mFile, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, ss.st_mode);
+  AutoFD ofd = ensure_open(mFile, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, ss.st_mode);
   if (ofd < 0)
     return WRITE_ERROR;
 
@@ -982,12 +1009,68 @@ PatchIfFile::Finish(int status)
 
 #ifdef XP_WIN
 #include "nsWindowsRestart.cpp"
+
+static void
+LaunchWinPostProcess(const char *appExe)
+{
+  // Launch helper.exe to perform post processing (e.g. registry and log file
+  // modifications) for the update.
+  char inifile[MAXPATHLEN];
+  strcpy(inifile, appExe);
+
+  char *slash = strrchr(inifile, '\\');
+  if (!slash)
+    return;
+
+  strcpy(slash + 1, "updater.ini");
+
+  char exefile[MAXPATHLEN];
+  char exearg[MAXPATHLEN];
+  if (!GetPrivateProfileString("PostUpdateWin", "ExeRelPath", NULL, exefile,
+      sizeof(exefile), inifile))
+    return;
+
+  if (!GetPrivateProfileString("PostUpdateWin", "ExeArg", NULL, exearg,
+      sizeof(exearg), inifile))
+    return;
+
+  char exefullpath[MAXPATHLEN];
+  strcpy(exefullpath, appExe);
+
+  slash = strrchr(exefullpath, '\\');
+  strcpy(slash + 1, exefile);
+
+  char dlogFile[MAXPATHLEN];
+  strcpy(dlogFile, exefullpath);
+
+  slash = strrchr(dlogFile, '\\');
+  strcpy(slash + 1, "uninstall.update");
+
+  char slogFile[MAXPATHLEN];
+  snprintf(slogFile, MAXPATHLEN, "%s/update.log", gSourcePath);
+
+  // We want to launch the post update helper app to update the Windows
+  // registry even if there is a failure with removing the uninstall.update
+  // file or copying the update.log file.
+  ensure_remove(dlogFile);
+  copy_file(slogFile, dlogFile);
+
+  static int    argc = 2;
+  static char **argv = (char**) malloc(sizeof(char*) * (argc + 1));
+  argv[0] = "argv0ignoredbywinlaunchchild";
+  argv[1] = exearg;
+  argv[2] = "\0";
+
+  WinLaunchChild(exefullpath, argc, argv, 1);
+  free(argv);
+}
 #endif
 
 static void
 LaunchCallbackApp(const char *workingDir, int argc, char **argv)
 {
   putenv("NO_EM_RESTART=");
+  putenv("MOZ_LAUNCHED_CHILD=1");
 
   // Run from the specified working directory (see bug 312360).
   chdir(workingDir);
@@ -997,7 +1080,7 @@ LaunchCallbackApp(const char *workingDir, int argc, char **argv)
 #elif defined(XP_MACOSX)
   LaunchChild(argc, argv);
 #elif defined(XP_WIN)
-  WinLaunchChild(argv[0], argc, argv);
+  WinLaunchChild(argv[0], argc, argv, -1);
 #else
 # warning "Need implementaton of LaunchCallbackApp"
 #endif
@@ -1011,7 +1094,7 @@ WriteStatusFile(int status)
   char filename[MAXPATHLEN];
   snprintf(filename, MAXPATHLEN, "%s/update.status", gSourcePath);
 
-  AutoFD fd = open(filename, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, 0644);
+  AutoFD fd = ensure_open(filename, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, 0644);
   if (fd < 0)
     return;
 
@@ -1091,19 +1174,6 @@ int main(int argc, char **argv)
 
   gSourcePath = argv[1];
 
-#ifdef XP_WIN
-  // By opening a file handle to the executable, we can tell the OS to
-  // prevent new application processes from launching while we are
-  // updating.
-
-  HANDLE exefile = NULL;
-
-  if (argc > 5)
-    exefile = CreateFile(argv[4], DELETE | GENERIC_WRITE,
-                         0, // no sharing!
-                         NULL, OPEN_EXISTING, 0, NULL);
-#endif
-
   LogInit();
 
   // Run update process on a background thread.  ShowProgressUI may return
@@ -1117,8 +1187,8 @@ int main(int argc, char **argv)
   LogFinish();
 
 #ifdef XP_WIN
-  if (exefile)
-    CloseHandle(exefile);
+  if (gSucceeded && argc > 4)
+    LaunchWinPostProcess(argv[4]);
 #endif
 
   // The callback to execute is given as the last N arguments of our command
@@ -1172,6 +1242,14 @@ ActionList::Append(Action *action)
 int
 ActionList::Prepare()
 {
+  // If the action list is empty then we should fail in order to signal that
+  // something has gone wrong. Otherwise we report success when nothing is
+  // actually done. See bug 327140.
+  if (mCount == 0) {
+    LOG(("empty action list\n"));
+    return UNEXPECTED_ERROR;
+  }
+
   Action *a = mFirst;
   while (a) {
     int rv = a->Prepare();
@@ -1218,6 +1296,11 @@ ActionList::Finish(int status)
     a = a->mNext;
   }
 
+#ifdef XP_WIN
+  if (status == OK)
+    gSucceeded = TRUE;
+#endif
+
   UpdateProgressUI(100.0f);
 }
 
