@@ -247,9 +247,8 @@ nsLocalFile::nsLocalFile() :
 }
 
 nsLocalFile::nsLocalFile(const nsLocalFile& other)
-  : mCachedStat(other.mCachedStat)
-  , mPath(other.mPath)
-  , mHaveCachedStat(other.mHaveCachedStat)
+  : mPath(other.mPath)
+  , mHaveCachedStat(PR_FALSE)
 {
 }
 
@@ -1248,9 +1247,11 @@ nsLocalFile::GetParent(nsIFile **aParent)
  */
 
 
-#ifdef XP_BEOS
+#if defined(XP_BEOS) || defined(SOLARIS)
 // access() is buggy in BeOS POSIX implementation, at least for BFS, using stat() instead
 // see bug 169506, https://bugzilla.mozilla.org/show_bug.cgi?id=169506
+// access() problem also exists in Solaris POSIX implementation
+// see bug 351595, https://bugzilla.mozilla.org/show_bug.cgi?id=351595
 NS_IMETHODIMP
 nsLocalFile::Exists(PRBool *_retval)
 {
@@ -1299,6 +1300,11 @@ nsLocalFile::IsExecutable(PRBool *_retval)
     NS_ENSURE_ARG_POINTER(_retval);
     struct stat buf;
 
+    if (IsDesktopFile()) {
+        *_retval = PR_TRUE;
+        return NS_OK;
+    }
+    
     *_retval = (stat(mPath.get(), &buf) == 0);
     if (*_retval || errno == EACCES) {
         *_retval = *_retval && (buf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH ));
@@ -1349,6 +1355,11 @@ nsLocalFile::IsExecutable(PRBool *_retval)
     CHECK_mPath();
     NS_ENSURE_ARG_POINTER(_retval);
 
+    if (IsDesktopFile()) {
+        *_retval = PR_TRUE;
+        return NS_OK;
+    }
+    
     *_retval = (access(mPath.get(), X_OK) == 0);
     if (*_retval || errno == EACCES)
         return NS_OK;
@@ -1774,3 +1785,13 @@ void
 nsLocalFile::GlobalShutdown()
 {
 }
+
+PRBool
+nsLocalFile::IsDesktopFile()
+{
+    // Just needs to be good enough to match nsFileProtocolHandler::ReadURLFile
+    nsCAutoString leafName;
+    nsresult rv = GetNativeLeafName(leafName);
+    return NS_FAILED(rv) ||
+        StringEndsWith(leafName, NS_LITERAL_CSTRING(".desktop"));
+}
