@@ -65,6 +65,7 @@
 #include "nsInstallResources.h"
 #include "nsIProxyObjectManager.h"
 #include "nsIWindowWatcher.h"
+#include "nsIAuthPrompt.h"
 #include "nsIWindowMediator.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsDirectoryService.h"
@@ -120,17 +121,12 @@ nsXPInstallManager::nsXPInstallManager()
 
     // initialize mLastUpdate to the current time
     mLastUpdate = PR_Now();
-
-    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
-    if (os)
-        os->AddObserver(this, XPI_PROGRESS_TOPIC, PR_TRUE);
 }
 
 
 nsXPInstallManager::~nsXPInstallManager()
 {
-    if (mTriggers)
-        delete mTriggers;
+    NS_ASSERTION(!mTriggers, "Shutdown not called, triggers still alive");
 }
 
 
@@ -173,6 +169,12 @@ nsXPInstallManager::InitManagerWithHashes(const PRUnichar **aURLs,
     if (!mTriggers)
         return NS_ERROR_OUT_OF_MEMORY;
 
+    mNeedsShutdown = PR_TRUE;
+
+    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+    if (os)
+        os->AddObserver(this, XPI_PROGRESS_TOPIC, PR_TRUE);
+
     for (PRUint32 i = 0; i < aURLCount; ++i) 
     {
         nsXPITriggerItem* item = new nsXPITriggerItem(0, aURLs[i], nsnull,
@@ -181,6 +183,7 @@ nsXPInstallManager::InitManagerWithHashes(const PRUnichar **aURLs,
         {
             delete mTriggers; // nsXPITriggerInfo frees any alloc'ed nsXPITriggerItems
             mTriggers = nsnull;
+            Shutdown();
             return NS_ERROR_OUT_OF_MEMORY;
         }
         mTriggers->Add(item);
@@ -192,10 +195,14 @@ nsXPInstallManager::InitManagerWithHashes(const PRUnichar **aURLs,
     {
         delete mTriggers;
         mTriggers = nsnull;
+        Shutdown();
         return rv;
     }
 
-    return Observe(aListener, XPI_PROGRESS_TOPIC, NS_LITERAL_STRING("open").get());
+    rv = Observe(aListener, XPI_PROGRESS_TOPIC, NS_LITERAL_STRING("open").get());
+    if (NS_FAILED(rv))
+        Shutdown();
+    return rv;
 }
 
 
@@ -211,9 +218,9 @@ nsXPInstallManager::InitManager(nsIScriptGlobalObject* aGlobalObject, nsXPITrigg
 
     nsresult rv = NS_OK;
 
+    mNeedsShutdown = PR_TRUE;
     mTriggers = aTriggers;
     mChromeType = aChromeType;
-    mNeedsShutdown = PR_TRUE;
 
     mParentWindow = do_QueryInterface(aGlobalObject);
 
@@ -279,24 +286,35 @@ nsXPInstallManager::InitManagerInternal()
         // Get permission to install
         //-----------------------------------------------------
 
+#ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
         if ( mChromeType == CHROME_SKIN )
         {
+            // We may want to enable the simple installation UI once
+            // bug 343037 is fixed
+
             // skins get a simpler/friendlier dialog
             // XXX currently not embeddable
             OKtoInstall = ConfirmChromeInstall( mParentWindow, packageList );
         }
         else
         {
+#endif
             rv = dlgSvc->ConfirmInstall( mParentWindow,
                                          packageList,
                                          numStrings,
                                          &OKtoInstall );
             if (NS_FAILED(rv))
                 OKtoInstall = PR_FALSE;
+#ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
         }
+#endif
 
         if (OKtoInstall)
         {
+            nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+            if (os)
+                os->AddObserver(this, XPI_PROGRESS_TOPIC, PR_TRUE);
+
             //-----------------------------------------------------
             // Open the progress dialog
             //-----------------------------------------------------
@@ -383,6 +401,7 @@ nsXPInstallManager::ConfirmInstall(nsIDOMWindow *aParent, const PRUnichar **aPac
     return rv;
 }
 
+#ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
 PRBool nsXPInstallManager::ConfirmChromeInstall(nsIDOMWindowInternal* aParentWindow, const PRUnichar **aPackage)
 {
     // get the dialog strings
@@ -440,7 +459,7 @@ PRBool nsXPInstallManager::ConfirmChromeInstall(nsIDOMWindowInternal* aParentWin
 
     return bInstall;
 }
-
+#endif
 
 NS_IMETHODIMP
 nsXPInstallManager::OpenProgressDialog(const PRUnichar **aPackageList, PRUint32 aCount, nsIObserver *aObserver)
@@ -551,11 +570,11 @@ NS_IMETHODIMP nsXPInstallManager::Observe( nsISupports *aSubject,
                             do_GetService(kProxyObjectManagerCID, &rv);
                 if (pmgr)
                 {
-                    rv = pmgr->GetProxyForObject( NS_UI_THREAD_EVENTQ,
-                                                  NS_GET_IID(nsIXPIProgressDialog),
-                                                  dlg,
-                                                  PROXY_SYNC | PROXY_ALWAYS,
-                                                  getter_AddRefs(mDlg) );
+                    pmgr->GetProxyForObject( NS_UI_THREAD_EVENTQ,
+                                             NS_GET_IID(nsIXPIProgressDialog),
+                                             dlg,
+                                             PROXY_SYNC | PROXY_ALWAYS,
+                                             getter_AddRefs(mDlg) );
                 }
             }
 
@@ -858,7 +877,28 @@ void nsXPInstallManager::Shutdown()
 
         nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
         if (os)
-            os->RemoveObserver(this, XPI_PROGRESS_TOPIC);
+        {
+            nsresult rv;
+            nsCOMPtr<nsIProxyObjectManager> pmgr =
+                        do_GetService(kProxyObjectManagerCID, &rv);
+            if (pmgr)
+            {
+                nsCOMPtr<nsIObserverService> pos;
+                rv = pmgr->GetProxyForObject( NS_UI_THREAD_EVENTQ,
+                                              NS_GET_IID(nsIObserverService),
+                                              os,
+                                              PROXY_SYNC | PROXY_ALWAYS,
+                                              getter_AddRefs(pos) );
+                if (NS_SUCCEEDED(rv))
+                    pos->RemoveObserver(this, XPI_PROGRESS_TOPIC);
+            }
+        }
+
+        if (mTriggers)
+        {
+            delete mTriggers;
+            mTriggers = nsnull;
+        }
 
         NS_RELEASE_THIS();
     }
@@ -912,7 +952,7 @@ nsXPInstallManager::GetDestinationFile(nsString& url, nsILocalFile* *file)
             if (NS_SUCCEEDED(rv))
             { 
                 temp->AppendNative(NS_LITERAL_CSTRING("tmp.xpi"));
-                temp->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0644);
+                temp->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
                 *file = temp;
                 NS_IF_ADDREF(*file);
             }
@@ -962,6 +1002,18 @@ nsXPInstallManager::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
 {
     nsresult rv = NS_ERROR_FAILURE;
 
+    // If we are dealing with a HTTP request, then treat HTTP error pages as
+    // download failures.
+    nsCOMPtr<nsIHttpChannel> httpChan = do_QueryInterface(request);
+    if (httpChan) {
+        PRBool succeeded;
+        if (NS_SUCCEEDED(httpChan->GetRequestSucceeded(&succeeded)) && !succeeded) {
+            // HTTP response is not a 2xx!
+            request->Cancel(NS_BINDING_ABORTED);
+            return NS_OK;
+        }
+    }
+
     NS_ASSERTION( mItem && mItem->mFile, "XPIMgr::OnStartRequest bad state");
     if ( mItem && mItem->mFile )
     {
@@ -970,7 +1022,7 @@ nsXPInstallManager::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
         rv = NS_NewLocalFileOutputStream(getter_AddRefs(mItem->mOutStream),
                                          mItem->mFile,
                                          PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE,
-                                         0664);
+                                         0600);
     }
     return rv;
 }
@@ -1120,6 +1172,22 @@ nsXPInstallManager::OnStatus(nsIRequest* request, nsISupports *ctxt,
 NS_IMETHODIMP
 nsXPInstallManager::GetInterface(const nsIID & eventSinkIID, void* *_retval)
 {
+    if (eventSinkIID.Equals(NS_GET_IID(nsIAuthPrompt))) {    
+        *_retval = nsnull;
+ 
+        nsresult rv;
+        nsCOMPtr<nsIWindowWatcher> ww(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
+        NS_ENSURE_SUCCESS(rv, rv);
+ 
+        nsCOMPtr<nsIAuthPrompt> prompt;
+        rv = ww->GetNewAuthPrompter(nsnull, getter_AddRefs(prompt));
+        NS_ENSURE_SUCCESS(rv, rv);
+ 
+        nsIAuthPrompt *p = prompt.get();
+        NS_ADDREF(p);
+        *_retval = p;
+        return NS_OK;
+    }
     return QueryInterface(eventSinkIID, (void**)_retval);
 }
 
